// Neural Space Rev. 2002
// By Hector Lopez
// 10/3/2002


#include "NSView.h"
#include "Defs.h"


NSView::NSView(BRect rect, const char *name) : BView(rect, name, 
			B_FOLLOW_ALL_SIDES, B_WILL_DRAW | B_PULSE_NEEDED | B_FRAME_EVENTS) {

	popupMenu = new BPopUpMenu("PopUpMenu", false, false, B_ITEMS_IN_COLUMN);
	popupMenu->SetFont(be_plain_font);
	configItem = new BMenuItem("Preferences", new BMessage(NS_PREFS));
	aboutItem = new BMenuItem("About...", new BMessage(NS_ABOUT));
	quitItem = new BMenuItem("Quit", new BMessage(NS_QUIT));
	
	popupMenu->AddItem(configItem);
	
	popupMenu->AddSeparatorItem();
	
	popupMenu->AddItem(aboutItem);
	popupMenu->AddItem(quitItem);
	
	frame_color.red = (NS_DEFAULT_FRAME_COLOR & 0xff000000) >> 24;
	frame_color.green = (NS_DEFAULT_FRAME_COLOR & 0x00ff0000) >> 16;
	frame_color.blue = (NS_DEFAULT_FRAME_COLOR & 0x0000ff00) >> 8;
	
	SetViewColor(frame_color);
	
	active_color.red = (NS_DEFAULT_ACTIVE_COLOR & 0xff000000) >> 24;
	active_color.green = (NS_DEFAULT_ACTIVE_COLOR & 0x00ff0000) >> 16;
	active_color.blue = (NS_DEFAULT_ACTIVE_COLOR & 0x0000ff00) >> 8;
	
	idle_color.red = (NS_DEFAULT_IDLE_COLOR & 0xff000000) >> 24;
	idle_color.green = (NS_DEFAULT_IDLE_COLOR & 0x00ff0000) >> 16;
	idle_color.blue = (NS_DEFAULT_IDLE_COLOR & 0x0000ff00) >> 8;
	
	messageRunner = NULL;
	prefsWindow = NULL;
};

NSView::NSView(BMessage *mes) : BView(mes) {
	SetResizingMode(B_FOLLOW_ALL_SIDES);
	SetFlags(B_WILL_DRAW | B_PULSE_NEEDED | B_FRAME_EVENTS);
	
	popupMenu = new BPopUpMenu("PopUpMenu", false, false, B_ITEMS_IN_COLUMN);
	popupMenu->SetFont(be_plain_font);
	configItem = new BMenuItem("Preferences", new BMessage(NS_PREFS));
	aboutItem = new BMenuItem("About...", new BMessage(NS_ABOUT));
	quitItem = new BMenuItem("Quit", new BMessage(NS_QUIT));
	
	popupMenu->AddItem(configItem);
	
	popupMenu->AddSeparatorItem();
	
	popupMenu->AddItem(aboutItem);
	popupMenu->AddItem(quitItem);
	
	int8 temp;
	
	mes->FindInt8("frameRed", &temp);
	frame_color.red = (uint8) temp;
	mes->FindInt8("frameGreen", &temp);
	frame_color.green = (uint8) temp;
	mes->FindInt8("frameBlue", &temp);
	frame_color.blue = (uint8) temp;
	
	SetViewColor(frame_color);
	
	mes->FindInt8("idleRed", &temp);
	idle_color.red = (uint8) temp;
	mes->FindInt8("idleGreen", &temp);
	idle_color.green = (uint8) temp;
	mes->FindInt8("idleBlue", &temp);
	idle_color.blue = (uint8) temp;
	
	mes->FindInt8("activeRed", &temp);
	active_color.red = (uint8) temp;
	mes->FindInt8("activeGreen", &temp);
	active_color.green = (uint8) temp;
	mes->FindInt8("activeBlue", &temp);
	active_color.blue = (uint8) temp;
		
	messageRunner = NULL;
	prefsWindow = NULL;
};

void NSView::AttachedToWindow() {
	configItem->SetTarget(this);
	aboutItem->SetTarget(this);
	quitItem->SetTarget(this);
	
	messenger = BMessenger(this);
	messageRunner = new BMessageRunner(messenger, new BMessage(NS_PULSE), 250000);
};

BArchivable *NSView::Instantiate(BMessage *data) {
	if (!validate_instantiation(data, "NSView")) return NULL;
	return (new NSView(data));
};

NSView::~NSView() {
	if (messageRunner != NULL) delete messageRunner;
	
	popupMenu->RemoveItem(configItem);
	popupMenu->RemoveItem(aboutItem);
	popupMenu->RemoveItem(quitItem);
	if (popupMenu != NULL) delete popupMenu;
	
	if (configItem != NULL) delete configItem;
	if (aboutItem != NULL) delete aboutItem;
	if (quitItem != NULL) delete quitItem;
};

void NSView::MouseDown(BPoint point) {
	BPoint cursor;
	uint32 buttons;
	MakeFocus(true);
	GetMouse(&cursor, &buttons, true);
	
	if (buttons & B_SECONDARY_MOUSE_BUTTON) {
		ConvertToScreen(&point);
		point = point - BPoint(2,2);
		popupMenu->Go(point, true, false, true);
	}
};

status_t NSView::Archive(BMessage *data, bool deep) const {
	BView::Archive(data, deep);
	data->AddString("class", "NSView");
	
	data->AddInt8("frameRed", frame_color.red);
	data->AddInt8("frameGreen", frame_color.green);
	data->AddInt8("frameBlue", frame_color.blue);
	
	data->AddInt8("idleRed", idle_color.red);
	data->AddInt8("idleGreen", idle_color.green);
	data->AddInt8("idleBlue", idle_color.blue);
	
	data->AddInt8("activeRed", active_color.red);
	data->AddInt8("activeGreen", active_color.green);
	data->AddInt8("activeBlue", active_color.blue);
	
	return B_OK;
};

void NSView::Update() {
	system_info sysinfo;
	get_system_info(&sysinfo);
	
	memoryLoad = ((float) sysinfo.used_pages) / ((float) sysinfo.max_pages);
};

void NSView::Draw(BRect frame) {
	
	SetDrawingMode(B_OP_COPY);

	BRect frameSize = Bounds();
	int height = frameSize.IntegerHeight() - 2;

	SetHighColor(frame_color);
	FillRect(frameSize);
	
	SetHighColor(active_color);
	int activeHeight = (int)(memoryLoad * (height + 1));
	if (activeHeight > height) activeHeight = height;
	
	double reminder = memoryLoad * (height + 1) - activeHeight;
	
	rgb_color fraction_color;
	fraction_color.red = (uint8)(idle_color.red + reminder * (active_color.red - idle_color.red));
	fraction_color.green = (uint8)(idle_color.green + reminder * (active_color.green - idle_color.green));
	fraction_color.blue = (uint8)(idle_color.blue + reminder * (active_color.blue - idle_color.blue));
	fraction_color.alpha = 0xff;
	
	int idleHeight = height - activeHeight;
	if (idleHeight > 0) {
		SetHighColor(idle_color);
		BRect idleFrame(1, 1, frameSize.IntegerWidth()-1, idleHeight);
		FillRect(idleFrame);
	};
	
	if (activeHeight > 0) {
		SetHighColor(fraction_color);
		FillRect(BRect(1, frameSize.IntegerHeight()-activeHeight-1, frameSize.IntegerWidth()-1, frameSize.IntegerHeight()-activeHeight-1));
		SetHighColor(active_color);
		FillRect(BRect(1, frameSize.IntegerHeight()-activeHeight, frameSize.IntegerWidth()-1, frameSize.IntegerHeight()-1));
	};
};

void NSView::RemoveReplicant() {
	BDeskbar *the_deskbar = new BDeskbar();
	the_deskbar->RemoveItem("Neural Space Rev2002");
	delete the_deskbar;
};

void NSView::ShowAbout() {
	BAlert *alert = new BAlert("Neural Space Rev2002", "Neural Space Revision 2002\n\nBy Hector Lopez\nBased on Pulse", 
									"Ok", NULL, NULL,  B_WIDTH_AS_USUAL, B_OFFSET_SPACING);
	alert->Go(NULL);
};

void NSView::MessageReceived(BMessage *msg) {
	switch (msg->what) {
		case NS_PREFS: {
			passing_things someData;
			someData.frame = frame_color;
			someData.idle = idle_color;
			someData.active = active_color;
			BRect temp = Bounds();
			someData.width = temp.IntegerWidth();
			someData.msgTarget = &messenger;
			
			if (prefsWindow != NULL) {
				prefsWindow->Activate(true);
				break;
			}
			prefsWindow = new ConfigWindow(BRect(100,100,390,230), "Neural Space Revision 2002 Preferences",
				someData);
			prefsWindow->Show();
			break;
		};
		
		case NS_ABOUT:
			ShowAbout();
			break;
		
		case NS_QUIT:	
			RemoveReplicant();
			break;
		
		case NS_PULSE:	
			Update();
			Draw(Bounds());
			break;
			
		case NS_RESET:
			frame_color.red = (NS_DEFAULT_FRAME_COLOR & 0xff000000) >> 24;
			frame_color.green = (NS_DEFAULT_FRAME_COLOR & 0x00ff0000) >> 16;
			frame_color.blue = (NS_DEFAULT_FRAME_COLOR & 0x0000ff00) >> 8;
	
			active_color.red = (NS_DEFAULT_ACTIVE_COLOR & 0xff000000) >> 24;
			active_color.green = (NS_DEFAULT_ACTIVE_COLOR & 0x00ff0000) >> 16;
			active_color.blue = (NS_DEFAULT_ACTIVE_COLOR & 0x0000ff00) >> 8;
	
			idle_color.red = (NS_DEFAULT_IDLE_COLOR & 0xff000000) >> 24;
			idle_color.green = (NS_DEFAULT_IDLE_COLOR & 0x00ff0000) >> 16;
			idle_color.blue = (NS_DEFAULT_IDLE_COLOR & 0x0000ff00) >> 8;
			
			ResizeTo(8, 15);
			break;
			
		case NS_ICON_WIDTH_CONTROL:
			int8 new_icon_width;
			msg->FindInt8("width", &new_icon_width);
			ResizeTo(new_icon_width, 15);
			break;
			
		case NS_COLOR_CHANGE:
			int32 temp_color;
			status_t found;
			
			if ((found = msg->FindInt32("active_color", &temp_color)) == B_OK) {
				active_color.red = (temp_color & 0xff000000) >> 24;
				active_color.green = (temp_color & 0x00ff0000) >> 16;
				active_color.blue = (temp_color & 0x0000ff00) >> 8;
			};
			if ((found = msg->FindInt32("idle_color", &temp_color)) == B_OK) {
				idle_color.red = (temp_color & 0xff000000) >> 24;
				idle_color.green = (temp_color & 0x00ff0000) >> 16;
				idle_color.blue = (temp_color & 0x0000ff00) >> 8;
			};
			if ((found = msg->FindInt32("frame_color", &temp_color)) == B_OK) {
				frame_color.red = (temp_color & 0xff000000) >> 24;
				frame_color.green = (temp_color & 0x00ff0000) >> 16;
				frame_color.blue = (temp_color & 0x0000ff00) >> 8;
			};
			break;
		
		case NS_PREFS_WINDOW_DIED:
			prefsWindow = NULL;
			break;
		
		default:
			BView::MessageReceived(msg);
			break;
	};
};